class InvalidRomanNumberException extends Exception {

}

class Roman {

	public final static int MIN = 1;
	public final static int MAX = 3999;

	// Datenstruktur:
	// klassenintern werden roemische Zahlen als int-Werte verwaltet
	int value;

	public Roman(String roman) throws InvalidRomanNumberException {
		this.value = Roman.intValue(roman);
	}

	public Roman(int value) throws InvalidRomanNumberException {
		if (value < Roman.MIN || value > Roman.MAX) {
			throw new InvalidRomanNumberException();
		}
		this.value = value;
	}

	public Roman(Roman roman) {
		this.value = roman.value;
	}

	@Override
	public Roman clone() {
		return new Roman(this);
	}

	@Override
	public String toString() {
		return Roman.toRoman(this.value);
	}

	@Override
	public boolean equals(Object roman) {
		return this.value == ((Roman) roman).value;
	}

	public Roman add(Roman roman) throws InvalidRomanNumberException {
		if (this.value + roman.value > Roman.MAX) {
			throw new InvalidRomanNumberException();
		}
		this.value += roman.value;
		return this;
	}

	public static Roman add(Roman r1, Roman r2)
			throws InvalidRomanNumberException {
		return new Roman(Roman.toRoman(r1.value + r2.value));
	}

	// interne Datenstruktur fuer Umrechnungszwecke
	static String symbol = "IVXLCDM";
	static int[] wert = { 1, 5, 10, 50, 100, 500, 1000 };

	static int intValue(String roman) throws InvalidRomanNumberException {
		int value = 0;
		// String der Reihe nach durcharbeiten
		for (int i = 0; i < roman.length(); i++) {
			// Index bestimmen
			int index1 = Roman.symbol.indexOf(roman.charAt(i));
			if (index1 == -1) {
				throw new InvalidRomanNumberException();
			}
			if (i + 1 < roman.length()) { // es folgen weitere Symbole
				int index2 = Roman.symbol.indexOf(roman.charAt(i + 1));
				if (index2 == -1) {
					throw new InvalidRomanNumberException();
				}
				if (index1 < index2) { // Differenz bilden und addieren
					value = value + Roman.wert[index2] - Roman.wert[index1];
					i++;
				} else { // addieren
					value = value + Roman.wert[index1];
				}
			} else { // letztes Symbol
				value = value + Roman.wert[index1];
			}
		}
		if (value > Roman.MAX) {
			throw new InvalidRomanNumberException();
		}
		return value;
	}

	// interne Datenstruktur fuer Umrechnungszwecke
	static String symbols[][] = {
			// Symbole fuer 1-9
			{ "", "I", "II", "III", "IV", "V", "VI", "VII", "VIII", "IX" },
			// Symbole fuer 10-90
			{ "", "X", "XX", "XXX", "XL", "L", "LX", "LXX", "LXXX", "XC" },
			// Symbole fuer 100-900
			{ "", "C", "CC", "CCC", "CD", "D", "DC", "DCC", "DCCC", "CM" },
			// Symbol fuer 1000
			{ "", "M", "MM", "MMM" } };

	static String toRoman(int value) {
		String roman = new String("");
		int potenz = 0;
		while (value > 0) {
			// bestimme letzte Ziffer
			int ziffer = value % 10;
			// erweitere die Roemische Zahl entsprechend der Ziffer und der
			// aktuellen Stelligkeit
			roman = Roman.symbols[potenz][ziffer] + roman;
			// erhoehe Stelligkeit
			potenz++;
			// kuerze value um eine Stelle
			value = value / 10;
		}
		return roman;
	}
}

public class UE21Aufgabe12 {

	// Testprogramm
	public static void main(String[] args) throws InvalidRomanNumberException {
		while (true) {
			int value = IO.readInt("int-Wert eingeben: ");
			Roman roman1 = new Roman(value);
			IO.println(roman1.toString());

			String eingabe = IO.readString("Roemische Zahl eingeben: ");
			Roman roman2 = new Roman(eingabe);
			IO.println(roman2.toString());

			Roman roman3 = Roman.add(roman1, roman2);
			IO.println("Summe: " + roman3);
		}
	}

}
