import java.util.ArrayList;
import java.util.HashMap;

class UnknownNodeException extends Exception {
}

class NodeCollection extends ArrayList<String> {
}

class Graph {

	private HashMap<String, NodeCollection> graph;

	public Graph() {
		graph = new HashMap<String, NodeCollection>();
	}

	public Graph(String... nodes) {
		graph = new HashMap<String, NodeCollection>();
		for (String node : nodes) {
			addNode(node);
		}
	}

	public void addNode(String node) {
		if (!graph.containsKey(node)) {
			graph.put(node, new NodeCollection());
		}
	}

	public void addEdge(String fromNode, String toNode)
			throws UnknownNodeException {
		checkNode(fromNode);
		checkNode(toNode);
		NodeCollection targets = graph.get(fromNode);
		targets.add(toNode);
	}

	public boolean existsDirectConnection(String fromNode, String toNode)
			throws UnknownNodeException {
		checkNode(fromNode);
		checkNode(toNode);
		NodeCollection targets = graph.get(fromNode);
		return targets.contains(toNode);
	}

	public boolean existsConnection(String fromNode, String toNode)
			throws UnknownNodeException {
		checkNode(fromNode);
		checkNode(toNode);
		return existsConnection(fromNode, toNode, new ArrayList<String>());
	}

	private boolean existsConnection(String fromNode, String toNode,
			ArrayList<String> visitedNodes) {

		NodeCollection targets = graph.get(fromNode);
		if (targets.contains(toNode)) {
			return true;
		}
		visitedNodes.add(fromNode);
		for (String target : targets) {
			if (!visitedNodes.contains(target)
					&& existsConnection(target, toNode, visitedNodes)) {
				return true;
			}
		}
		return false;
	}

	private void checkNode(String node) throws UnknownNodeException {
		if (!graph.containsKey(node)) {
			throw new UnknownNodeException();
		}
	}
}

public class UE21Aufgabe22 {

	public static void main(String[] args) throws Exception {
		Graph graph = new Graph("1", "2", "3", "4", "5");
		graph.addEdge("1", "2");
		graph.addEdge("2", "3");
		graph.addEdge("3", "2");
		graph.addEdge("3", "4");
		graph.addEdge("1", "5");
		System.out.println(graph.existsConnection("1", "4"));
		System.out.println(graph.existsConnection("2", "5"));
	}

}


