import java.util.ArrayList;

class Property {
	private String key;
	private String value;

	Property(String key, String value) {
		this.key = key;
		this.value = value;
	}

	public boolean equals(Object obj) {
		Property p = (Property) obj;
		return this.key.equals(p.key);
	}

	String getKey() {
		return this.key;
	}

	String getValue() {
		return this.value;
	}
}

class PropertyNotDefinedException extends Exception {
}

/**
 * verwaltet eine Liste mit Properties
 */
class PropertyList {

	private PropertyList defaults;
	private ArrayList<Property> propertyList;

	/**
	 * Konstruktor mit einem Properties-Objekt als Parameter fr das gilt: Wird
	 * beim Aufruf der Methode getProperty kein Property mit dem angegebenen
	 * Namen gefunden, wird im Properties-Objekt parent weitergesucht (ggfls.
	 * rekursive Fortsetzung dieses Prozesses bei dessen "parent")
	 * 
	 * @param defaults
	 *            "Parent"-PropertyListe
	 */
	public PropertyList(PropertyList parent) {
		this.defaults = parent;
		this.propertyList = new ArrayList<Property>();
	}

	/**
	 * Default-Konstruktor: erzeugt eine leere Property-Liste ohne
	 * "Parent"-PropertyListe
	 */
	public PropertyList() {
		this(null);
	}

	/**
	 * existiert bereits ein Property mit dem key, wird ihm der neue Wert value
	 * zugeordnet; andernfalls wird ein neues Property (key/value) erzeugt und
	 * gespeichert
	 * 
	 * @param key
	 *            Name des Properties
	 * @param value
	 *            Wert des Properties
	 */
	public void setProperty(String key, String value) {
		Property prop = new Property(key, value);
		if (this.propertyList.contains(prop)) {
			this.propertyList.remove(prop);
		}
		this.propertyList.add(prop);
	}

	/**
	 * liefert den dem Property mit dem Namen key zugeordneten Wert; existiert
	 * kein entsprechendes Property, wird eine Exception geworfen
	 * 
	 * @param key
	 *            Name des Property
	 * @return der dem Namen zugeordnete Wert
	 * @throws PropertyNotDefinedException
	 *             wird geworfen, falls kein Property mit dem Namen key
	 *             existiert
	 */
	public String getProperty(String key) throws PropertyNotDefinedException {
		Property prop = new Property(key, null);
		int index = this.propertyList.indexOf(prop);
		if (index == -1) {
			if (this.defaults != null) {
				return this.defaults.getProperty(key);
			} else {
				throw new PropertyNotDefinedException();
			}
		} else {
			prop = this.propertyList.get(index);
			return prop.getValue();
		}
	}
}

public class UE21Aufgabe24 {

	public static void main(String[] args) {
		PropertyList standard = new PropertyList();
		standard.setProperty("1", "one");
		standard.setProperty("2", "two");
		standard.setProperty("3", "three");
		standard.setProperty("4", "four");
		PropertyList p = new PropertyList(standard);
		p.setProperty("1", "eins");
		p.setProperty("2", "zwei");
		p.setProperty("3", "drei");
		try {
			System.out.println(p.getProperty("1"));
			System.out.println(p.getProperty("4"));
			System.out.println(p.getProperty("5"));
		} catch (PropertyNotDefinedException exc) {
			System.out.println("not defined");
		}

	}

}
