class InvalidHexaException extends Exception {

	protected String msg;

	public InvalidHexaException(char ch) {
		super();
		this.msg = "Ungueltiges Symbol: <" + ch + ">";
	}

	public InvalidHexaException(String str) {
		super();
		this.msg = str;
	}

	@Override
	public String toString() {
		return this.msg;
	}
}

/**
 * Klasse Hexa. Realisiert den Umgang mit positiven Hexadezimal-Zahlen.
 */
class Hexa extends java.lang.Number {

	// Datenstruktur
	protected final static int BASIS = 16;
	protected final static char[] ZIFFERN = { '0', '1', '2', '3', '4', '5',
			'6', '7', '8', '9', 'A', 'B', 'C', 'E', 'F' };

	/**
	 * int-Repraesentation der Hexadezimalzahl
	 */
	protected int value;

	// Konstruktoren
	/**
	 * initialisiert eine neue Hexadezimalzahl mit dem uebergebenen int-Wert
	 * 
	 * @param wert
	 *            int-Wert, mit dem die neue Hexadezimalzahl initialisiert
	 *            werden soll
	 * @exception InvalidHexaException
	 *                wird geworfen, wenn der Wert des aktuellen Parameters
	 *                negativ ist
	 */
	public Hexa(int wert) throws InvalidHexaException {
		super();
		if (wert < 0) {
			throw new InvalidHexaException("negativer Wert!");
		}
		this.value = wert;
	}

	/**
	 * initialisiert eine neue Hexadezimalzahl mit dem uebergebenen String-Wert
	 * 
	 * @param hexaString
	 *            String, mit dem die neue Hexadezimalzahl initialisiert werden
	 *            soll
	 * @exception InvalidHexaException
	 *                wird geworfen, wenn der String keine Hexadezimalzahl
	 *                repraesentiert
	 */
	public Hexa(String hexaString) throws InvalidHexaException {
		super();
		if (hexaString == null || hexaString.length() < 1) {
			throw new InvalidHexaException("leerer String!");
		}
		this.value = 0;
		try {
			int potenz = 1;
			for (int i = hexaString.length() - 1; i >= 0; i--) {
				this.value = this.value + Hexa.f(hexaString.charAt(i)) * potenz;
				potenz *= Hexa.BASIS;
			}
		} catch (StringIndexOutOfBoundsException obj) {
			// kann nicht auftreten
		}
	}

	/**
	 * Copy-Konstruktor
	 * 
	 * @param hexaObj
	 *            bereits existierendes Hexa-Objekt
	 */
	public Hexa(Hexa hexaObj) {
		super();
		this.value = hexaObj.value;
	}

	// geerbte und zu ueberschreibende Methoden

	/**
	 * ueberschreibt die von der Klasse Object geerbte Methode clone
	 * 
	 * @return neues Hexa-Objekt als Clone des aufgerufenen Objektes
	 */
	@Override
	protected Object clone() {
		return new Hexa(this);
	}

	/**
	 * ueberschreibt die von der Klasse Object geerbte Methode equals;
	 * 
	 * @param obj
	 *            ein bereits existierendes Hexa-Objekt
	 * @return liefert true, wenn die beiden Objekte denselben Hexadezimalwert
	 *         repraesentieren
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof Hexa)) {
			return false;
		}
		return this.value == ((Hexa) obj).value;
	}

	/**
	 * ueberschreibt die von der Klasse Object geerbte Methode toString;
	 * 
	 * @return liefert eine String-repraesentation des Hexadezimalwertes
	 */
	@Override
	public String toString() {
		if (this.value == 0) {
			return "0";
		}
		int help = this.value;
		StringBuffer buffer = new StringBuffer();
		while (help > 0) {
			buffer.append(Hexa.f_1(help % Hexa.BASIS));
			help /= Hexa.BASIS;
		}
		char[] reverse = new char[buffer.length()];
		try {
			for (int i = 0; i < buffer.length(); i++) {
				reverse[i] = buffer.charAt(buffer.length() - i - 1);
			}
		} catch (StringIndexOutOfBoundsException obj) {
			// kann nicht auftreten
		}
		return new String(reverse);
	}

	// Konvertierungsmethoden (in der Klasse Number abstract)
	@Override
	public double doubleValue() {
		return this.value;
	}

	@Override
	public float floatValue() {
		return this.value;
	}

	@Override
	public int intValue() {
		return this.value;
	}

	@Override
	public long longValue() {
		return this.value;
	}

	// Vergleichsmethoden
	public boolean kleiner(Hexa hexa_obj) {
		return this.value < hexa_obj.value;
	}

	public boolean kleinerGleich(Hexa hexa_obj) {
		return this.value <= hexa_obj.value;
	}

	public boolean groesser(Hexa hexa_obj) {
		return this.value > hexa_obj.value;
	}

	public boolean groesserGleich(Hexa hexa_obj) {
		return this.value >= hexa_obj.value;
	}

	public boolean gleich(Hexa hexa_obj) {
		return this.value == hexa_obj.value;
	}

	public boolean ungleich(Hexa hexa_obj) {
		return this.value != hexa_obj.value;
	}

	// Rechenmethoden
	public static Hexa addiere(Hexa wert_1, Hexa wert_2) {
		return new Hexa(wert_1).addiere(wert_2);
	}

	public static Hexa subtrahiere(Hexa von, Hexa was)
			throws InvalidHexaException {
		return new Hexa(von).subtrahiere(was);
	}

	public static Hexa multipliziere(Hexa wert_1, Hexa wert_2) {
		return new Hexa(wert_1).multipliziere(wert_2);
	}

	public static Hexa dividiere(Hexa wen, Hexa durch) throws DivNullException {
		return new Hexa(wen).dividiere(durch);
	}

	public static Hexa modulo(Hexa wen, Hexa durch) throws DivNullException {
		return new Hexa(wen).modulo(durch);
	}

	public Hexa addiere(Hexa wert) {
		this.value += wert.value;
		return new Hexa(this);
	}

	public Hexa subtrahiere(Hexa was) throws InvalidHexaException {
		if (this.value < was.value) {
			throw new InvalidHexaException("negative Zahl");
		}
		this.value -= was.value;
		return new Hexa(this);
	}

	public Hexa multipliziere(Hexa wert) {
		this.value *= wert.value;
		return new Hexa(this);
	}

	public Hexa dividiere(Hexa durch) throws DivNullException {
		if (durch.value == 0) {
			throw new DivNullException();
		}
		this.value /= durch.value;
		return new Hexa(this);
	}

	public Hexa modulo(Hexa durch) throws DivNullException {
		if (durch.value == 0) {
			throw new DivNullException();
		}
		this.value %= durch.value;
		return new Hexa(this);
	}

	final private static int f(char ch) throws InvalidHexaException {
		switch (ch) {
		case '0':
			return 0;
		case '1':
			return 1;
		case '2':
			return 2;
		case '3':
			return 3;
		case '4':
			return 4;
		case '5':
			return 5;
		case '6':
			return 6;
		case '7':
			return 7;
		case '8':
			return 8;
		case '9':
			return 9;
		case 'A':
			return 10;
		case 'B':
			return 11;
		case 'C':
			return 12;
		case 'D':
			return 13;
		case 'E':
			return 14;
		case 'F':
			return 15;
		default:
			throw new InvalidHexaException(ch);
		}
	}

	final private static char f_1(int value) {
		switch (value) {
		case 0:
			return '0';
		case 1:
			return '1';
		case 2:
			return '2';
		case 3:
			return '3';
		case 4:
			return '4';
		case 5:
			return '5';
		case 6:
			return '6';
		case 7:
			return '7';
		case 8:
			return '8';
		case 9:
			return '9';
		case 10:
			return 'A';
		case 11:
			return 'B';
		case 12:
			return 'C';
		case 13:
			return 'D';
		case 14:
			return 'E';
		case 15:
			return 'F';
		default:
			return '0';// kann nicht auftreten
		}
	}
}

public class UE21Aufgabe8 {
	public static void main(String[] args) {

	}
}
